@@ -18,16 +18,15 @@
 #include "progress.h"
 #include "audio.h"
 #include "config.h"
-#include "st_shared.h"
 #include "common.h"
 
 #include "game_common.h"
 
 #include "st_set.h"
-#include "st_over.h"
 #include "st_level.h"
 #include "st_start.h"
 #include "st_title.h"
+#include "st_shared.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -46,40 +45,45 @@ static int challenge_id;
 
 static void gui_level(int id, int i)
 {
-    const GLfloat *fore = 0, *back = 0;
+    struct level *l = get_level(i);
+
+    const GLubyte *fore = 0;
+    const GLubyte *back = 0;
 
     int jd;
 
-    if (!level_exists(i))
+    if (!l)
     {
         gui_label(id, " ", GUI_SML, GUI_ALL, gui_blk, gui_blk);
         return;
     }
 
-    if (level_opened(i))
+    if (level_opened(l))
     {
-        fore = level_bonus(i)     ? gui_grn : gui_wht;
-        back = level_completed(i) ? fore    : gui_yel;
+        fore = level_bonus(l)     ? gui_grn : gui_wht;
+        back = level_completed(l) ? fore    : gui_yel;
     }
 
-    jd = gui_label(id, level_name(i), GUI_SML, GUI_ALL, back, fore);
+    jd = gui_label(id, level_name(l), GUI_SML, GUI_ALL, back, fore);
 
-    if (level_opened(i) || config_cheat())
+    if (level_opened(l) || config_cheat())
         gui_active(jd, i, 0);
 }
 
 static void start_over_level(int i)
 {
-    if (level_opened(i) || config_cheat())
-    {
-        gui_set_image(shot_id, level_shot(i));
+    struct level *l = get_level(i);
 
-        set_score_board(&get_level(i)->score.most_coins,  -1,
-                        &get_level(i)->score.best_times,  -1,
-                        &get_level(i)->score.fast_unlock, -1);
+    if (level_opened(l) || config_cheat())
+    {
+        gui_set_image(shot_id, level_shot(l));
+
+        set_score_board(level_score(l, SCORE_COIN), -1,
+                        level_score(l, SCORE_TIME), -1,
+                        level_score(l, SCORE_GOAL), -1);
 
         if (file_id)
-            gui_set_label(file_id, level_file(i));
+            gui_set_label(file_id, level_file(l));
     }
 }
 
@@ -99,8 +103,8 @@ static void start_over(int id, int pulse)
     {
         gui_set_image(shot_id, set_shot(curr_set()));
 
-        set_score_board(set_coin_score(curr_set()), -1,
-                        set_time_score(curr_set()), -1,
+        set_score_board(set_score(curr_set(), SCORE_COIN), -1,
+                        set_score(curr_set(), SCORE_TIME), -1,
                         NULL, -1);
     }
 
@@ -134,9 +138,9 @@ static int start_action(int i)
         }
         break;
 
-    case GUI_MOST_COINS:
-    case GUI_BEST_TIMES:
-    case GUI_FAST_UNLOCK:
+    case GUI_SCORE_COIN:
+    case GUI_SCORE_TIME:
+    case GUI_SCORE_GOAL:
         gui_score_set(i);
         return goto_state(&st_start);
 
@@ -149,7 +153,7 @@ static int start_action(int i)
         return goto_state(&st_start);
 
     default:
-        if (progress_play(i))
+        if (progress_play(get_level(i)))
             return goto_state(&st_level);
         break;
     }
@@ -157,7 +161,7 @@ static int start_action(int i)
     return 1;
 }
 
-static int start_enter(void)
+static int start_gui(void)
 {
     int w = config_get_d(CONFIG_WIDTH);
     int h = config_get_d(CONFIG_HEIGHT);
@@ -165,8 +169,6 @@ static int start_enter(void)
 
     int id, jd, kd, ld;
 
-    progress_init(MODE_NORMAL);
-
     if ((id = gui_vstack(0)))
     {
         if ((jd = gui_hstack(id)))
@@ -211,9 +213,9 @@ static int start_enter(void)
             }
         }
         gui_space(id);
-        gui_score_board(id, (GUI_MOST_COINS |
-                             GUI_BEST_TIMES |
-                             GUI_FAST_UNLOCK), 0, 0);
+        gui_score_board(id, (GUI_SCORE_COIN |
+                             GUI_SCORE_TIME |
+                             GUI_SCORE_GOAL), 0, 0);
         gui_space(id);
 
         if ((jd = gui_hstack(id)))
@@ -244,12 +246,22 @@ static int start_enter(void)
 
         gui_layout(id, 0, 0);
 
+        if (file_id)
+            gui_set_trunc(file_id, TRUNC_HEAD);
+
         set_score_board(NULL, -1, NULL, -1, NULL, -1);
     }
 
+    return id;
+}
+
+static int start_enter(struct state *st, struct state *prev)
+{
+    progress_init(MODE_NORMAL);
+
     audio_music_fade_to(0.5f, "bgm/inter.ogg");
 
-    return id;
+    return start_gui();
 }
 
 static void start_point(int id, int x, int y, int dx, int dy)
@@ -257,12 +269,9 @@ static void start_point(int id, int x, int y, int dx, int dy)
     start_over(gui_point(id, x, y), 1);
 }
 
-static void start_stick(int id, int a, int v)
+static void start_stick(int id, int a, float v, int bump)
 {
-    int x = (config_tst_d(CONFIG_JOYSTICK_AXIS_X, a)) ? v : 0;
-    int y = (config_tst_d(CONFIG_JOYSTICK_AXIS_Y, a)) ? v : 0;
-
-    start_over(gui_stick(id, x, y), 1);
+    start_over(gui_stick(id, a, v, bump), 1);
 }
 
 static int start_keybd(int c, int d)
@@ -339,6 +348,5 @@ struct state st_start = {
     shared_angle,
     shared_click,
     start_keybd,
-    start_buttn,
-    1, 0
+    start_buttn
 };
