@@ -36,12 +36,12 @@ static int time_btn_id;
 static int goal_btn_id;
 
 static void set_score_color(int id, int hi,
-                            const GLfloat *c0,
-                            const GLfloat *c1)
+                            const GLubyte *c0,
+                            const GLubyte *c1)
 {
-    if (hi >= 0)
+    if (hi >= RANK_HARD)
     {
-        if (hi < NSCORE)
+        if (hi < RANK_LAST)
             gui_set_color(id, c0, c0);
         else
             gui_set_color(id, c1, c1);
@@ -81,7 +81,7 @@ static void gui_scores(int id, int e)
             {
                 if ((md = gui_vstack(ld)))
                 {
-                    for (j = 0; j < NSCORE - 1; j++)
+                    for (j = RANK_HARD; j < RANK_EASY; j++)
                         score_coin[j] = gui_count(md, 1000, GUI_SML, 0);
 
                     score_coin[j++] = gui_count(md, 1000, GUI_SML, GUI_SE);
@@ -95,7 +95,7 @@ static void gui_scores(int id, int e)
 
                 if ((md = gui_vstack(ld)))
                 {
-                    for (j = 0; j < NSCORE; j++)
+                    for (j = RANK_HARD; j < RANK_LAST; j++)
                     {
                         score_name[j] = gui_label(md, s, GUI_SML, 0,
                                                   gui_yel, gui_wht);
@@ -113,7 +113,7 @@ static void gui_scores(int id, int e)
 
                 if ((md = gui_vstack(ld)))
                 {
-                    for (j = 0; j < NSCORE - 1; j++)
+                    for (j = RANK_HARD; j < RANK_EASY; j++)
                         score_time[j] = gui_clock(md, 359999, GUI_SML, 0);
 
                     score_time[j++] = gui_clock(md, 359999, GUI_SML, GUI_SW);
@@ -135,13 +135,13 @@ static void gui_scores(int id, int e)
 static void gui_set_scores(const char *label, const struct score *s, int hilite)
 {
     const char *name;
-    int j;
+    int j, n = score_extra_row ? RANK_LAST : RANK_EASY;
 
     if (s == NULL)
     {
         gui_set_label(score_label, _("Unavailable"));
 
-        for (j = 0; j < NSCORE + score_extra_row ; j++)
+        for (j = RANK_HARD; j <= n; j++)
         {
             gui_set_count(score_coin[j], -1);
             gui_set_label(score_name[j], "");
@@ -152,7 +152,7 @@ static void gui_set_scores(const char *label, const struct score *s, int hilite)
     {
         gui_set_label(score_label, label);
 
-        for (j = 0; j < NSCORE + score_extra_row; j++)
+        for (j = RANK_HARD; j <= n; j++)
         {
             name = s->player[j];
 
@@ -170,15 +170,15 @@ static void gui_set_scores(const char *label, const struct score *s, int hilite)
 
 /*---------------------------------------------------------------------------*/
 
-static int score_type = GUI_MOST_COINS;
+static int score_type = GUI_SCORE_COIN;
 
 void gui_score_board(int pd, unsigned int types, int e, int h)
 {
     int id, jd, kd, ld;
 
-    assert((types & GUI_MOST_COINS)  == GUI_MOST_COINS ||
-           (types & GUI_BEST_TIMES)  == GUI_BEST_TIMES ||
-           (types & GUI_FAST_UNLOCK) == GUI_FAST_UNLOCK );
+    assert((types & GUI_SCORE_COIN)  == GUI_SCORE_COIN ||
+           (types & GUI_SCORE_TIME)  == GUI_SCORE_TIME ||
+           (types & GUI_SCORE_GOAL) == GUI_SCORE_GOAL );
 
     /* Make sure current score type matches the spec. */
 
@@ -197,23 +197,23 @@ void gui_score_board(int pd, unsigned int types, int e, int h)
             {
                 gui_filler(kd);
 
-                if ((types & GUI_MOST_COINS) == GUI_MOST_COINS)
+                if ((types & GUI_SCORE_COIN) == GUI_SCORE_COIN)
                 {
                     coin_btn_id = gui_state(kd, _("Most Coins"),
-                                            GUI_SML, GUI_MOST_COINS,
-                                            score_type == GUI_MOST_COINS);
+                                            GUI_SML, GUI_SCORE_COIN,
+                                            score_type == GUI_SCORE_COIN);
                 }
-                if ((types & GUI_BEST_TIMES) == GUI_BEST_TIMES)
+                if ((types & GUI_SCORE_TIME) == GUI_SCORE_TIME)
                 {
                     time_btn_id = gui_state(kd, _("Best Times"),
-                                            GUI_SML, GUI_BEST_TIMES,
-                                            score_type == GUI_BEST_TIMES);
+                                            GUI_SML, GUI_SCORE_TIME,
+                                            score_type == GUI_SCORE_TIME);
                 }
-                if ((types & GUI_FAST_UNLOCK) == GUI_FAST_UNLOCK)
+                if ((types & GUI_SCORE_GOAL) == GUI_SCORE_GOAL)
                 {
                     goal_btn_id = gui_state(kd, _("Fast Unlock"),
-                                            GUI_SML, GUI_FAST_UNLOCK,
-                                            score_type == GUI_FAST_UNLOCK);
+                                            GUI_SML, GUI_SCORE_GOAL,
+                                            score_type == GUI_SCORE_GOAL);
                 }
 
                 if (h)
@@ -240,28 +240,28 @@ void gui_score_board(int pd, unsigned int types, int e, int h)
     }
 }
 
-void set_score_board(const struct score *smc, int hmc,
-                     const struct score *sbt, int hbt,
-                     const struct score *sfu, int hfu)
+void set_score_board(const struct score *sc, int hc,
+                     const struct score *st, int ht,
+                     const struct score *sg, int hg)
 {
     switch (score_type)
     {
-    case GUI_MOST_COINS:
-        gui_set_scores(_("Most Coins"), smc, hmc);
+    case GUI_SCORE_COIN:
+        gui_set_scores(_("Most Coins"), sc, hc);
         break;
 
-    case GUI_BEST_TIMES:
-        gui_set_scores(_("Best Times"), sbt, hbt);
+    case GUI_SCORE_TIME:
+        gui_set_scores(_("Best Times"), st, ht);
         break;
 
-    case GUI_FAST_UNLOCK:
-        gui_set_scores(_("Fast Unlock"), sfu, hfu);
+    case GUI_SCORE_GOAL:
+        gui_set_scores(_("Fast Unlock"), sg, hg);
         break;
     }
 
-    set_score_color(coin_btn_id, hmc, gui_grn, gui_wht);
-    set_score_color(time_btn_id, hbt, gui_grn, gui_wht);
-    set_score_color(goal_btn_id, hfu, gui_grn, gui_wht);
+    set_score_color(coin_btn_id, hc, gui_grn, gui_wht);
+    set_score_color(time_btn_id, ht, gui_grn, gui_wht);
+    set_score_color(goal_btn_id, hg, gui_grn, gui_wht);
 }
 
 void gui_score_set(int t)
@@ -278,12 +278,12 @@ int  gui_score_next(int t)
 {
     switch (t)
     {
-    case GUI_MOST_COINS:  return GUI_BEST_TIMES;
-    case GUI_BEST_TIMES:  return GUI_FAST_UNLOCK;
-    case GUI_FAST_UNLOCK: return GUI_MOST_COINS;
+    case GUI_SCORE_COIN:  return GUI_SCORE_TIME;
+    case GUI_SCORE_TIME:  return GUI_SCORE_GOAL;
+    case GUI_SCORE_GOAL: return GUI_SCORE_COIN;
 
     default:
-        return GUI_MOST_COINS;
+        return GUI_SCORE_COIN;
     }
 }
 
