@@ -1,14 +1,14 @@
 #ifndef GEOM_H
 #define GEOM_H
 
-/*---------------------------------------------------------------------------*/
-
-#include "solid.h"
+#include "solid_draw.h"
 
 /*---------------------------------------------------------------------------*/
 
 #define IMG_SHAD "png/shadow.png"
 
+#define BACK_DIST   256.0f
+#define FAR_DIST    512.0f
 #define JUMP_HEIGHT   2.00f
 #define SWCH_HEIGHT   2.00f
 #define GOAL_HEIGHT   3.00f
@@ -16,39 +16,28 @@
 
 /*---------------------------------------------------------------------------*/
 
-void mark_init(int);
-void mark_free(void);
-void mark_draw(void);
+void geom_init(void);
+void geom_free(void);
+
+void goal_draw(struct s_rend *, float);
+void jump_draw(struct s_rend *, float, int);
+void swch_draw(struct s_rend *, int, int);
+void flag_draw(struct s_rend *);
+void mark_draw(struct s_rend *);
+void vect_draw(struct s_rend *);
+void back_draw(struct s_rend *, float);
 
 /*---------------------------------------------------------------------------*/
 
-void goal_init(int);
-void goal_free(void);
-void goal_draw(void);
-
-/*---------------------------------------------------------------------------*/
-
-void jump_init(int);
-void jump_free(void);
-void jump_draw(int);
-
-/*---------------------------------------------------------------------------*/
-
-void swch_init(int);
-void swch_free(void);
-void swch_draw(int, int);
-
-/*---------------------------------------------------------------------------*/
-
-void flag_init(int);
-void flag_free(void);
-void flag_draw(void);
+void back_init(const char *s);
+void back_free(void);
+void back_draw_easy(void);
 
 /*---------------------------------------------------------------------------*/
 
 void shad_init(void);
 void shad_free(void);
-void shad_draw_set(const float *, float);
+void shad_draw_set(void);
 void shad_draw_clr(void);
 
 /*---------------------------------------------------------------------------*/
