@@ -105,6 +105,9 @@ static GLuint make_texture(const void *p, int w, int h, int b)
 {
     static const GLenum format[] =
         { 0, GL_LUMINANCE, GL_LUMINANCE_ALPHA, GL_RGB, GL_RGBA };
+    static const GLenum type[] =
+        { 0, GL_UNSIGNED_BYTE, GL_UNSIGNED_BYTE,
+            GL_UNSIGNED_SHORT_5_6_5, GL_UNSIGNED_SHORT_4_4_4_4 };
 
     GLuint o = 0;
 
@@ -113,17 +116,17 @@ static GLuint make_texture(const void *p, int w, int h, int b)
 #ifdef GL_TEXTURE_MAX_ANISOTROPY_EXT
     int a = config_get_d(CONFIG_ANISO);
 #endif
+#ifdef GL_GENERATE_MIPMAP_SGIS
     int m = config_get_d(CONFIG_MIPMAP);
+#endif
     int k = config_get_d(CONFIG_TEXTURES);
     int W = w;
     int H = h;
 
-    GLint max;
+    GLint max = gli.max_texture_size;
 
     void *q = NULL;
 
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max);
-
     while (w / k > (int) max || h / k > (int) max)
         k *= 2;
 
@@ -135,8 +138,8 @@ static GLuint make_texture(const void *p, int w, int h, int b)
     glGenTextures(1, &o);
     glBindTexture(GL_TEXTURE_2D, o);
 
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
@@ -157,7 +160,7 @@ static GLuint make_texture(const void *p, int w, int h, int b)
 
     glTexImage2D(GL_TEXTURE_2D, 0,
                  format[b], W, H, 0,
-                 format[b], GL_UNSIGNED_BYTE, q ? q : p);
+                 format[b], type[b], q ? q : p);
 
     if (q) free(q);
 
@@ -165,6 +168,41 @@ static GLuint make_texture(const void *p, int w, int h, int b)
     return o;
 }
 
+static void *image_to_16bpp(unsigned char *data, int w, int h, int b)
+{
+    unsigned char *p, *row;
+    int i, j;
+
+    if (b <= 2) return data;
+
+    if ((p = malloc(w * h * 2)))
+    {
+        for (i = 0; i < h; i++)
+        {
+            row = data + i * w * b;
+            for (j = 0; j < w; j++)
+            {
+                unsigned short packed, *pp;
+                if (b == 4)
+                    packed =
+                        (row[j * b] >> 4) << 12 |
+                        (row[j * b + 1] >> 4) << 8 |
+                        (row[j * b + 2] >> 4) << 4 |
+                        row[j * b + 3] >> 4;
+                else
+                    packed =
+                        (row[j * b] >> 3) << 11 |
+                        (row[j * b + 1] >> 2) << 5 |
+                        (row[j * b + 2] >> 3);
+                pp = (unsigned short *)&p[i * w * 2 + j * 2];
+                *pp = packed;
+            }
+        }
+    }
+    free(data);
+    return p;
+}
+
 /*
  * Load an image from the named file.  Return an OpenGL texture object.
  */
@@ -180,6 +218,7 @@ GLuint make_image_from_file(const char *filename)
 
     if ((p = image_load(filename, &w, &h, &b)))
     {
+        p = image_to_16bpp(p, w, h, b);
         o = make_texture(p, w, h, b);
         free(p);
     }
@@ -202,7 +241,7 @@ GLuint make_image_from_font(int *W, int *H,
 
     /* Render the text. */
 
-    if (text && strlen(text) > 0)
+    if (font && text && strlen(text) > 0)
     {
         SDL_Color    col = { 0xFF, 0xFF, 0xFF, 0xFF };
         SDL_Surface *orig;
@@ -251,6 +290,7 @@ GLuint make_image_from_font(int *W, int *H,
 
             /* Create the OpenGL texture object. */
 
+            p = image_to_16bpp(p, w2, h2, b);
             o = make_texture(p, w2, h2, b);
 
             free(p);
@@ -284,30 +324,13 @@ SDL_Surface *load_surface(const char *filename)
 
     SDL_Surface *srf = NULL;
 
-    Uint32 rmask;
-    Uint32 gmask;
-    Uint32 bmask;
-    Uint32 amask;
-
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-    rmask = 0xFF000000;
-    gmask = 0x00FF0000;
-    bmask = 0x0000FF00;
-    amask = 0x000000FF;
-#else
-    rmask = 0x000000FF;
-    gmask = 0x0000FF00;
-    bmask = 0x00FF0000;
-    amask = 0xFF000000;
-#endif
-
     if ((p = image_load(filename, &w, &h, &b)))
     {
         void *q;
 
         if ((q = image_flip(p, w, h, b, 0, 1)))
             srf = SDL_CreateRGBSurfaceFrom(q, w, h, b * 8, w * b,
-                                           rmask, gmask, bmask, amask);
+                                           RMASK, GMASK, BMASK, AMASK);
         free(p);
     }
     return srf;
